close all
clear all
clc

dir_name = './EstNews_KK_4/metropolis/';

% Loading MH Results
load('./EstNews_KK_4_results.mat')

% Loading MH history
load('./EstNews_KK_4/metropolis/EstNews_KK_4_mh_history_0')

eval(['fileNames = dir(''' dir_name '*blck*'');'])

nMHfiles = size(fileNames,1);

nDraws = record.MhDraws(1);
nParameters = length(bayestopt_.name);

pos_eta_ = find(strcmp('eta_', bayestopt_.name));
pos_theta_ = find(strcmp('theta_', bayestopt_.name));
pos_sigma = find(strcmp('sigma', bayestopt_.name));
pos_gz2 = find(strcmp('gz2', bayestopt_.name));
pos_h = find(strcmp('h', bayestopt_.name));

Draws = zeros(nDraws, nParameters); 
line_start = 1;

for i = 1:nMHfiles
    % load an MH file
    eval(['load ' dir_name 'EstNews_KK_4_mh' num2str(i) '_blck1.mat'])
    nn = size(x2,1);
    line_end = line_start + nn - 1;
    Draws(line_start:line_end,:) = x2;
    line_start = line_end + 1;
end

alpha = 0.36;
beta = 0.985;
sc = 0.78;
muc_ss = 1.15;

burnin = options_.mh_drop;

sigma = Draws(:,pos_sigma);
h = Draws(:,pos_h);
gz = Draws(:,pos_gz2)/400;
theta_ = Draws(:,pos_theta_);
eta_ = Draws(:,pos_eta_);

gc = exp( gz/(1-alpha) );
beta_ = gc.^(-1./sigma).*h.*beta;
h_ = h./gc; 

omegaN = (1 - beta_)./(1 - h_).*(1 - sigma).*(1 - alpha)./(sc*muc_ss);

comove = theta_ + omegaN - eta_;
comove(1:burnin*nDraws,:) = [];

% Computing prior distributions
[a, b] = FindBetaPar(0.5, 0.2^2);
prior_sigma = betarnd(a, b, nDraws*10, 1);
prior_h = betarnd(a, b, nDraws*10, 1);
[a, b] = FindGPar(1.65, 1^2);
prior_gz = gamrnd(a, b, nDraws*10, 1)/400;
[a, b] = FindGPar(1, 0.5^2);
prior_theta_ = gamrnd(a, b, nDraws*10, 1);
[a, b] = FindGPar(1, 0.25^2);
prior_eta_ = gamrnd(a, b, nDraws*10, 1);

prior_gc = exp( prior_gz/(1-alpha) );
prior_beta_ = prior_gc.^(-1./prior_sigma).*prior_h.*beta;
prior_h_ = prior_h./prior_gc; 

prior_omegaN = (1 - prior_beta_)./(1 - prior_h_).*(1 - prior_sigma).*(1 - alpha)./(sc*muc_ss);
prior_comove = prior_theta_ + prior_omegaN - prior_eta_;
prior_comove(1:burnin*nDraws*10,:) = [];

% -- Taken from Dynare files
number_of_grid_points = 2^9;
bandwidth = 0;                    % Rule of thumb optimal bandwidth parameter.
kernel_function = 'gaussian';     % Gaussian kernel for Fast Fourrier Transform approximaton.  

prior_xx = sort(prior_comove);
number_of_draws = length(prior_xx);
optimal_bandwidth = mh_optimal_bandwidth(prior_xx,number_of_draws,bandwidth,kernel_function);
[prior_density(:,1),prior_density(:,2)] = kernel_density_estimate(prior_xx,number_of_grid_points,number_of_draws,optimal_bandwidth,kernel_function);

xx = sort(comove);
number_of_draws = length(xx);
optimal_bandwidth = mh_optimal_bandwidth(xx,number_of_draws,bandwidth,kernel_function);
[density(:,1),density(:,2)] = kernel_density_estimate(xx,number_of_grid_points,number_of_draws,optimal_bandwidth,kernel_function);

% --

figure
subplot(2,1,1)
fh = plot(density(:,1), density(:,2));
set(fh, 'LineWidth', 2)
hold on
fh = plot(prior_density(:,1), prior_density(:,2));
legend('Posterior', 'Prior')
set(fh, 'LineWidth', 2)
set(gca, 'XLim', [-1, 4])
eval(['disp({''Prior: Mean = ' num2str(mean(prior_comove)) ', 90% Interval = [' num2str(prctile(prior_comove, 5)) ', ' num2str(prctile(prior_comove, 95))  ']''; ''Posterior: Mean = ' num2str(mean(comove)) ', 90% Interval = [' num2str(prctile(comove, 5)) ', ' num2str(prctile(comove, 95))  ']''});'])
